#include <stdio.h>
#include <string.h>

/*
  Napomena: tekstualne datoteke na većini operativnih sustava
  na samom kraju datoteke sadrže prelazak u novi red (znak \n).

  Primjerice, ako imamo datoteke:

  1.txt
  123#include <2.txt>456

  2.txt
  Ja sam 2.txt

  Onda će izlaz (uz n = 1) izgledati ovako:
  123Ja sam 2.txt
  456

  To je ispravno rješenje; 2.txt (uz konvencije uobičajenih operativnih sustava)
  nakon "Ja sam 2.txt" sadrži '\n', i taj prelazak u novi red je ispravno
  kopiran u izlaz.
*/

void ispis(int n) {
    char *include_string = "#include";

    /* znak koji trenutno citamo */
    int znak;

    /* koliko smo znakova naredbe "#include" procitali
       prije citanja trenutnog znaka */
    int znakova = 0;

    /* konstruiramo ime glavne ulazne datoteke i otvaramo ju;
       odmah otvaramo i izlaznu datoteku */
    FILE *ulaz_dat, *izlaz_dat;
    char ulaz_adresa[] = "n.txt";
    char izlaz_adresa[] = "sve.txt";

    ulaz_adresa[0] = n + '0'; // elegantnije: sprintf
    ulaz_dat = fopen(ulaz_adresa, "r");
    izlaz_dat = fopen(izlaz_adresa, "w");

    /* čitamo znak po znak glavne ulazne datoteke (n.txt) */
    while ((znak = fgetc(ulaz_dat)) != EOF) {
        /* jesmo li već pročitali neki prefiks stringa "#include"? */
        if (znakova > 0) {
            if (znak == include_string[znakova])
                ++znakova;
            else {
                /* pročitali smo prefiks stringa "#include" ali smo naišli
                   na znak koji nije trebao biti na tom mjestu;
                   stoga moramo ispisati taj prefiks u izlaznu datoteku */
                int i;
                for (i = 0; i < znakova; ++i)
                    fputc(include_string[i], izlaz_dat);
                znakova = 0;
            }

            /* jesmo li pročitali sve znakove stringa "#include"? */
            if (znakova == strlen(include_string)) {
                /* dalje čitamo ime pomoćne datoteke
                   (znamo da je sintaksa ispravna, ne provjeravamo) */

                char ukljucena_adresa[] = "n.txt";
                FILE *pomocna_dat;

                fgetc(ulaz_dat); fgetc(ulaz_dat); /* razmak, simbol < */
                ukljucena_adresa[0] = fgetc(ulaz_dat); /* prvo slovo početne datoteke */

                /* preskačemo ".txt>" */
                fgetc(ulaz_dat); fgetc(ulaz_dat); fgetc(ulaz_dat);
                fgetc(ulaz_dat); fgetc(ulaz_dat);

                /* pročitali smo naredbu #include i imamo ime pomoćne datoteke,
                   otvaramo ju i prepisujemo u izlaznu datoteku */
                pomocna_dat = fopen(ukljucena_adresa, "r");
                while ((znak = fgetc(pomocna_dat)) != EOF)
                    fputc(znak, izlaz_dat);
                fclose(pomocna_dat);

                /* nastavljamo s čitanjem izvorne datoteke */
                znakova = 0;
                znak = fgetc(ulaz_dat);
            }
        }

        /* bitno je da nemamo "else" ispred "if (znakova == 0)" jer
           smo u tijelu gornjeg if bloka (ako se izvršio) pročitali znak koji
           još nije obrađen */
        if (znakova == 0) {
           if (znak == include_string[0])
                znakova = 1;
           else
                fputc(znak, izlaz_dat);
        }
    }
    fclose(izlaz_dat);
    fclose(ulaz_dat);
}

/*
  #include direktive i funkcija main nisu dio rješenja, služe samo za testiranje
*/
int main() {
    ispis(1);
}
